package edg.constraint;

public class ListConstraint extends AccessConstraint
{
	public enum Position { H, T, S, HS }
	// H -> Head
	// T -> Tail
	// S -> Structure: Interested in the list structure but not in its elements (Only positive constraints)
		// -[]T can be traversed but does not consume the +[]S
		// -[]H cannot be traversed

	protected final Position position;

	public ListConstraint(Operation operation)
	{
		this(operation, null);
	}
	public ListConstraint(Operation operation, Position position)
	{
		super(operation, AccessConstraint.CompositeType.List);

		this.position = position;
	}

	public Position getPosition()
	{
		return this.position;
	}

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof ListConstraint))
			return false;

		final ListConstraint constraint = (ListConstraint) object;

		if (!super.equals(constraint))
			return false;
		return this.position == constraint.position;
	}
	public String toString()
	{
		if (this.position != null)
			return super.toString() + this.position;
		return super.toString();
	}

	public ListConstraint opposite()
	{
		if (this.operation == Operation.Add)
			return new ListConstraint(Operation.Remove, this.position);
		return new ListConstraint(Operation.Add, this.position);
	}

	public boolean letThrough(AccessConstraint topConstraint){
		if (!(topConstraint instanceof ListConstraint))
			return false;
		ListConstraint top = (ListConstraint) topConstraint;
		if (this.position == Position.T && top.position == Position.S)
			return true;
		return false;
	}
}