package edg.constraint;

import edg.graph.EDG;
import edg.graph.Edge;
import edg.slicing.Phase;
import edg.constraint.AccessConstraint.Operation;

import java.util.List;

public class StructuralConstraint extends EdgeConstraint {
    private AccessConstraint constraint;

    public StructuralConstraint(Operation operation, String index) {
        constraint = new DataConstructorConstraint(operation, index + "S");
    }

    public StructuralConstraint(Operation operation, ListConstraint.Position position) {
        constraint = new ListConstraint(operation, ListConstraint.Position.HS);
    }

    private StructuralConstraint(AccessConstraint constraint) {
        this.constraint = constraint;
    }

    public AccessConstraint getConstraint() {
        return this.constraint;
    }

    public boolean equals(Object object) {
        if (object == this)
            return true;
        if (!(object instanceof StructuralConstraint))
            return false;

        final StructuralConstraint sConstraint = (StructuralConstraint) object;
        return this.constraint.equals(sConstraint.getConstraint());
    }

    public String toString() {
        return this.constraint.toString();
    }

    public boolean cancels(StructuralConstraint constraint){
        return this.equals(constraint.opposite());
    }

    public StructuralConstraint opposite() {
        final AccessConstraint opposite = this.constraint.opposite();
        return new StructuralConstraint(opposite);
    }

    protected List<Constraints> resolve(Phase phase, EDG edg, Edge edge, Constraints constraints, int productionDepth) {
        if (this.constraint.operation == Operation.Add)
            return super.wrap(super.push(phase, constraints));
        super.check(this.constraint.operation, Operation.Remove);
        if (phase.isInstanceof(Phase.Slicing))
            return super.wrap();
        super.check(phase, Phase.SummaryGeneration);
        return super.wrap(super.push(phase, constraints));
    }


    protected List<Constraints> resolve(Phase phase, EDG edg, Edge edge, Constraints constraints, AccessConstraint topConstraint, int productionDepth) {
        throw new IllegalStateException("This combination of constraints is illegal");
    }
    protected List<Constraints> resolve(Phase phase, EDG edg, Edge edge, Constraints constraints, StructuralConstraint topConstraint, int productionDepth) {
        if (topConstraint.constraint.operation == Operation.Add)
        {
            if (this.constraint.operation == Operation.Add)
                return super.wrap(super.push(phase, constraints));
            if (!constraints.nodeConstraints.isEmpty())
                throw new IllegalStateException("NodeConstraints cannot be stacked in this implementation");
            Constraint bottomConstraint = constraints.edgeConstraints.firstElement();
            if (!(bottomConstraint instanceof StructuralConstraint))
                throw new IllegalStateException("The bottom constraint must be a Structural Constraint");
            StructuralConstraint headConstraint = (StructuralConstraint) bottomConstraint;
            if (this.cancels(headConstraint)) {
                constraints.edgeConstraints.remove(0);
                return super.wrap(constraints);
            }
            return super.wrap();
        }

        super.check(topConstraint.constraint.operation, Operation.Remove);
        super.check(phase, Phase.SummaryGeneration);

        return super.wrap(super.push(phase, constraints));
    }
    protected List<Constraints> resolve(Phase phase, EDG edg, Edge edge, Constraints constraints, SeekingConstraint topConstraint, int productionDepth) {
        throw new IllegalStateException("This combination of constraints is illegal");
    }

    protected List<Constraints> resolve(Phase phase, EDG edg, Edge edge, Constraints constraints, AsteriskConstraint topConstraint, int productionDepth) {
        throw new IllegalStateException("This combination of constraints is illegal");
    }

    protected List<Constraints> resolve(Phase phase, EDG edg, Edge edge, Constraints constraints, GrammarConstraint topConstraint, int productionDepth) {
        throw new IllegalStateException("This combination of constraints is illegal");
    }
}