/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.work;

import edg.constraint.Constraints;
import edg.graph.EDG;
import edg.graph.Edge;
import edg.graph.Node;

public class EdgeWork extends Work
{
	private final Edge currentEdge;

	public EdgeWork(EDG edg, Node initialNode, Edge currentEdge, Constraints constraints)
	{
		super(initialNode, constraints);

		this.id = edg.getEdgeSource(currentEdge).getId() + "->" + edg.getEdgeTarget(currentEdge).getId();
		this.currentEdge = currentEdge;
	}

	public Edge getCurrentEdge()
	{
		return this.currentEdge;
	}

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof EdgeWork))
			return false;

		final EdgeWork edgeWork = (EdgeWork) object;

		if (!super.equals(edgeWork))
			return false;
		if (this.currentEdge != edgeWork.currentEdge)
			return false;
		return true;
	}
}