/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.work;

import edg.constraint.Constraints;
import edg.graph.Edge;
import edg.graph.Node;

public class NodeWork extends Work
{
	private final Node currentNode;
	private final Edge.Type previousEdgeType;

	public NodeWork(Node initialNode, Node currentNode, Constraints constraints)
	{
		super(initialNode, constraints);

		this.id = currentNode.getId() + "";
		this.currentNode = currentNode;
		this.previousEdgeType = null;
	}

	public NodeWork(Node initialNode, Node currentNode, Constraints constraints, Edge.Type type)
	{
		super(initialNode, constraints);

		this.id = currentNode.getId() + "";
		this.currentNode = currentNode;
		this.previousEdgeType = type;
	}

	public Node getCurrentNode()
	{
		return this.currentNode;
	}

	public Edge.Type getPreviousEdgeType() { return this.previousEdgeType; }

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof NodeWork))
			return false;

		final NodeWork nodeWork = (NodeWork) object;

		if (!super.equals(nodeWork))
			return false;
		return this.currentNode == nodeWork.currentNode &&
				previousEdgeType == nodeWork.previousEdgeType;
	}
}