/*
 * EDG, a library to generate and slice Expression Dependence Graphs.
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package edg.work;

import edg.constraint.Constraints;
import edg.graph.Node;

public abstract class Work
{
	protected String id;
	protected final Node initialNode;
	protected final Constraints constraints;

	public Work(Node initialNode, Constraints constraints)
	{
		this.initialNode = initialNode;
		this.constraints = constraints;
	}

	public String getId()
	{
		return this.id;
	}
	public Node getInitialNode()
	{
		return this.initialNode;
	}
	public Constraints getConstraints()
	{
		return this.constraints;
	}

	public boolean equals(Object object)
	{
		if (object == this)
			return true;
		if (!(object instanceof Work))
			return false;

		final Work work = (Work) object;

		// I think the formal out is not important
//		if (this.initialNode != work.initialNode)
//			return false;
		if (!this.constraints.equals(work.constraints))
			return false;
		return true;
	}
	public int hashCode()
	{
		return this.id.hashCode() + this.constraints.hashCode();
	}
}