/*
 * Miscellanea, helper classes for the EDG and e-Knife projects
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package misc;

import java.io.File;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;

public abstract class Config
{
	/********************************************************************************************************************************/
	/************************************************************ OBJECT ************************************************************/
	/********************************************************************************************************************************/
	protected final String programPath;

	/****************************************************************/
	/************************* Constructor **************************/
	/****************************************************************/
	public Config()
	{
		final String projectPath = Misc.getProjectPath(this.getClass());
		this.programPath = this.calculateProgramPath(projectPath);
	}
	private String calculateProgramPath(String projectPath)
	{
		String programPath = projectPath;

		final String jarEnding = "!" + File.separator;
		if (projectPath.endsWith(".jar" + jarEnding)) {
			programPath = programPath.substring(0, programPath.length() - jarEnding.length());
			programPath = programPath.substring(0, programPath.lastIndexOf(File.separator));
		}
		if (programPath.endsWith(File.separator + "bin") || programPath.endsWith(File.separator + "bin" + File.separator))
			programPath = programPath.substring(0, programPath.lastIndexOf("bin"));
		if (programPath.endsWith(File.separator + "lib") || programPath.endsWith(File.separator + "lib" + File.separator))
			programPath = programPath.substring(0, programPath.lastIndexOf("lib"));
		if (programPath.endsWith(File.separator + "Resources") || programPath.endsWith(File.separator + "Resources" + File.separator))
			programPath = programPath.substring(0, programPath.lastIndexOf("Resources"));

		programPath = programPath.endsWith(File.separator) ? programPath : programPath + File.separator;

		try
		{
			programPath = URLDecoder.decode(programPath, StandardCharsets.UTF_8);
		}
		catch (Exception ignored) {}

		return programPath;
	}
}
