/*
 * e-Knife, a program slicing tool for Erlang based on the EDG
 * Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package eknife.erlang;

import com.ericsson.otp.erlang.OtpErlangAtom;
import com.ericsson.otp.erlang.OtpErlangObject;

public class Launcher
{
	private static final Launcher launcher = new Launcher();
	public static Launcher getLauncher()
	{
		return Launcher.launcher;
	}

	private final ErlConnection erlConnection = new ErlConnection();

	private Launcher()
	{

	}

	public void open()
	{
		this.erlConnection.connect();
	}
	public void close()
	{
		this.erlConnection.disconnect();
	}

	public OtpErlangObject launch(String module, String function)
	{
		final OtpErlangObject[] arguments = new OtpErlangObject[0];

		return this.launch(module, function, arguments);
	}
	public OtpErlangObject launch(String module, String function, String... args)
	{
		final OtpErlangObject[] arguments = this.getArgs(args);

		return this.launch(module, function, arguments);
	}
	public OtpErlangObject launch(String module, String function, OtpErlangObject... args)
	{
		final boolean connected = this.erlConnection.isConnected();

		if (!connected)
			this.erlConnection.connect();
		this.erlConnection.send(module, function, args);
		final OtpErlangObject response = this.erlConnection.receive();
		if (!connected)
			this.erlConnection.disconnect();

		return response;
	}
	private OtpErlangObject[] getArgs(String... args)
	{
		final int argsLength = args.length;
		final OtpErlangObject[] arguments = new OtpErlangObject[argsLength];

		for (int argIndex = 0; argIndex < argsLength; argIndex++)
			arguments[argIndex] = new OtpErlangAtom(args[argIndex]);

		return arguments;
	}
}