%%
 % e-Knife, a program slicing tool for Erlang based on the EDG
 % Copyright (c) 2021. David Insa, Sergio Pérez, Josep Silva, Salvador Tamarit.
 %
 % This program is free software: you can redistribute it and/or modify
 % it under the terms of the GNU Affero General Public License as
 % published by the Free Software Foundation, either version 3 of the
 % License, or (at your option) any later version.
 %
 % This program is distributed in the hope that it will be useful,
 % but WITHOUT ANY WARRANTY; without even the implied warranty of
 % MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 % GNU Affero General Public License for more details.
 %
 % You should have received a copy of the GNU Affero General Public License
 % along with this program.  If not, see <https://www.gnu.org/licenses/>.
 %

-module(saver).
-export([save/1, save/2]).

save(ListsFileNameForms) ->
	save("./", ListsFileNameForms).

save(OutputFile, [{FileName, Forms}]) ->
	case filelib:is_dir(OutputFile) of
		true -> save_forms(OutputFile, {FileName, Forms});
		false -> save_form(OutputFile, Forms)
end;
save(OutputDir, ListsFilenameForms) ->
	lists:map(
		fun (FileNameForm) -> save_forms(OutputDir, FileNameForm) end,
		ListsFilenameForms).

save_forms(OutputDir, {Filename, Forms}) ->
	case filelib:is_dir(OutputDir) of
		true -> save_form(OutputDir ++ Filename, Forms);
		false -> throw(dir_provided_is_file)
	end.

save_form(OutputFile, Forms) ->
	file:write_file(OutputFile, list_to_binary(lists:flatten([lists:flatten(erl_pp:form(Form)) ++ "\n" || Form <- Forms]))).