%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------
%-- bench18.erl
%--
%-- AUTHORS:     Sebastian Danicic and John Howroyd
%-- DATE:        2002         
%-- PUBLISHED:   This program, known as Montréal Boat Example, was formulated by Sebastian 
%--              Danicic and John Howroyd during a boat excursion at the 2nd incarnation 
%--              of the SCAM conference.
%--              Source Code Analysis and Manipulation
%--              SCAM 2002 conference resources website
%--              http://www.ieee-scam.org/2002/Slides_ct.html
%--
%-- COPYLEFT:    This Erlang version is distributed in "Bencher: The Program Slicing 
%--              Benchmark Suite for Erlang" (Universitat Politècnica de València). 
%--              It is free of use and publicly available in the URL: 
%--              http://www.dsic.upv.es/~jsilva/slicing/bencher/
%--
%-- DESCRIPTION
%-- The program proposed by Danicic and Howroyd was designed to ilustrate an interesting 
%-- slicing problem. In this program redicates p and q, and functions f1, f2, and f3 
%-- depend only on their formal parameter. The relevant observation is that in any 
%-- terminating execution, the computation of K is irrelevant to the computation of J.
%-- (Description taken from "ORBS and the Limits of Static Slicing" and adapted to Erlang)
%--
%-- SLICING CRITERION: (50,J)
%-- SLICE: Available at: 
%--   http://www.dsic.upv.es/~jsilva/slicing/bencher/slices/b18_s50J.erl
%-----------------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------------
-module(bench18).
-export([mbe/2]).

-spec mbe(integer(),integer()) -> any().
mbe(J,K) ->
    case p(J) of
        true ->
            {NewJ,NewK} = 
                case q(K) of
                    true ->
                        TempK = f1(K),
                        {J,TempK};
                    false ->
                        TempK = f2(K),
                        TempJ = f3(J),
                        {TempJ,TempK}
            end,
            mbe(NewJ,NewK);
        false ->
            io:format("~p\n",[J])       % Slice here
    end.

p(J) ->
    J < 15.

q(K) ->
    K < 6.

f1(K) ->
    K + 2.

f2(K) ->
    K - 1.

f3(J) ->
    J + rand:uniform(5).